/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 * 
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 */

package com.simulogic.cese;

import java.util.Hashtable;
import java.util.prefs.Preferences;

import javax.swing.JLabel;

/**
 * Dialog that specify simulation options
 */
public class SimulationOptions extends javax.swing.JDialog {

	/** A return status code - returned if Cancel button has been pressed */
	public static final int RET_CANCEL = 0;

	/** A return status code - returned if OK button has been pressed */
	public static final int RET_OK = 1;

	/**
	 * A reference to Preferences object
	 */
	private Preferences simulationPreferences = null;

	private int iterations = 1;

	private int priority = Thread.NORM_PRIORITY;

	private boolean single = false;

	private boolean infinite = false;

	private boolean beep = false;

	private boolean history = false;

	private boolean highlight = false;

	/**
	 * Creates new form SimulationOptions
	 * 
	 * @param parent
	 *            parent frame
	 */
	public SimulationOptions(java.awt.Frame parent) {
		super(parent, true);

		initComponents();

		simulationPreferences = Preferences.userNodeForPackage(this.getClass());

		single = simulationPreferences.getBoolean("single", true);
		infinite = simulationPreferences.getBoolean("infinite", false);
		iterations = simulationPreferences.getInt("iterations", 1);
		priority = simulationPreferences.getInt("priority", 5);
		beep = simulationPreferences.getBoolean("beep", true);
		history = simulationPreferences.getBoolean("history", true);
		highlight = simulationPreferences.getBoolean("highlight", true);

		initParameters();
	}

	/** @return the return status of this dialog - one of RET_OK or RET_CANCEL */
	public int getReturnStatus() {
		return returnStatus;
	}

	private void initComponents() {
		buttonPanel = new javax.swing.JPanel();
		okButton = new javax.swing.JButton();
		cancelButton = new javax.swing.JButton();
		optPanel = new javax.swing.JPanel();
		threadPanel = new javax.swing.JPanel();
		itrPanel = new javax.swing.JPanel();
		numberPanel = new javax.swing.JPanel();
		threadSlider = new javax.swing.JSlider();
		simPanel = new javax.swing.JPanel();
		iterationLabel = new javax.swing.JLabel();
		iterationField = new com.simulogic.utils.IntegerField();
		infiniteCheckBox = new javax.swing.JCheckBox();
		radioPanel = new javax.swing.JPanel();
		singleRadioButton = new javax.swing.JRadioButton();
		continuousRadioButton = new javax.swing.JRadioButton();
		resPanel = new javax.swing.JPanel();
		beepCheckBox = new javax.swing.JCheckBox();
		historyCheckBox = new javax.swing.JCheckBox();
		highlightCheckBox = new javax.swing.JCheckBox();

		java.awt.GridBagConstraints gridBagConstraints;

		setTitle(ResourceManager.getResource("Simulation_Options"));
		setModal(true);
		addWindowListener(new java.awt.event.WindowAdapter() {

			public void windowClosing(java.awt.event.WindowEvent evt) {
				closeDialog();
			}
		});

		buttonPanel
				.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

		okButton.setText(ResourceManager.getResource("OK"));
		okButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(java.awt.event.ActionEvent evt) {
				okButtonActionPerformed();
			}
		});

		buttonPanel.add(okButton);

		cancelButton.setText(ResourceManager.getResource("Cancel"));
		cancelButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(java.awt.event.ActionEvent evt) {
				cancelButtonActionPerformed();
			}
		});

		buttonPanel.add(cancelButton);

		getContentPane().add(buttonPanel, java.awt.BorderLayout.SOUTH);

		optPanel.setLayout(new java.awt.GridBagLayout());

		threadPanel.setLayout(new javax.swing.BoxLayout(threadPanel,
				javax.swing.BoxLayout.X_AXIS));

		threadPanel.setBorder(new javax.swing.border.TitledBorder(
				new javax.swing.border.EtchedBorder(), ResourceManager
						.getResource("Sim_thread_priority")));
		threadSlider.setSnapToTicks(true);
		threadSlider.setPaintLabels(true);

		Hashtable labels = new Hashtable();
		labels.put(new Integer(Thread.MIN_PRIORITY), new JLabel("Min"));
		labels.put(new Integer(Thread.NORM_PRIORITY), new JLabel("Norm"));
		labels.put(new Integer(Thread.MAX_PRIORITY), new JLabel("Max"));
		threadSlider.setLabelTable(labels);

		threadSlider.setMinimum(Thread.MIN_PRIORITY);
		threadSlider.setMaximum(Thread.MAX_PRIORITY);
		threadSlider
				.setMajorTickSpacing((Thread.MAX_PRIORITY - Thread.MIN_PRIORITY) / 15);
		threadSlider
				.setMinorTickSpacing(threadSlider.getMajorTickSpacing() / 2);
		threadSlider.setPaintTicks(true);

		threadPanel.add(threadSlider);

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 0;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;

		optPanel.add(threadPanel, gridBagConstraints);
		simPanel.setBorder(new javax.swing.border.TitledBorder(
				new javax.swing.border.EtchedBorder(), ResourceManager
						.getResource("Sim_parameters")));

		radioPanel.setLayout(new javax.swing.BoxLayout(radioPanel,
				javax.swing.BoxLayout.Y_AXIS));

		radioPanel.setBorder(new javax.swing.border.TitledBorder(
				new javax.swing.border.EtchedBorder(), ResourceManager
						.getResource("Iterations")));
		singleRadioButton.setText(ResourceManager.getResource("Single"));
		singleRadioButton.setMnemonic('s');
		singleRadioButton
				.addActionListener(new java.awt.event.ActionListener() {

					public void actionPerformed(java.awt.event.ActionEvent evt) {
						singleRadioButtonActionPerformed();
					}
				});

		radioPanel.add(singleRadioButton);

		continuousRadioButton
				.setText(ResourceManager.getResource("Continuous"));
		continuousRadioButton.setMnemonic('c');
		continuousRadioButton
				.addActionListener(new java.awt.event.ActionListener() {

					public void actionPerformed(java.awt.event.ActionEvent evt) {
						singleRadioButtonActionPerformed();
					}
				});

		javax.swing.ButtonGroup group = new javax.swing.ButtonGroup();
		group.add(continuousRadioButton);
		group.add(singleRadioButton);
		radioPanel.add(continuousRadioButton);

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 0;
		simPanel.add(radioPanel, gridBagConstraints);

		numberPanel.setLayout(new java.awt.GridLayout(2, 1));

		iterationLabel.setText(ResourceManager
				.getResource("Number_of_iterations_"));
		iterationLabel.setDisplayedMnemonic('n');
		iterationLabel.setLabelFor(iterationField);
		itrPanel.add(iterationLabel);

		iterationField.setPositive(true);
		itrPanel.add(iterationField);

		numberPanel.add(itrPanel);

		infiniteCheckBox.setText(ResourceManager.getResource("Infinite_Loop"));
		infiniteCheckBox.setMnemonic('i');
		infiniteCheckBox.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(java.awt.event.ActionEvent evt) {
				infiniteCheckBoxActionPerformed();
			}
		});
		numberPanel.add(infiniteCheckBox);

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 1;
		gridBagConstraints.gridy = 0;
		simPanel.add(numberPanel, gridBagConstraints);

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
		optPanel.add(simPanel, gridBagConstraints);

		resPanel.setLayout(new javax.swing.BoxLayout(resPanel,
				javax.swing.BoxLayout.Y_AXIS));
		resPanel.setBorder(new javax.swing.border.TitledBorder(
				new javax.swing.border.EtchedBorder(), ResourceManager
						.getResource("After_simulation_run")));

		beepCheckBox.setText(ResourceManager.getResource("Beep"));
		beepCheckBox.setMnemonic('b');
		resPanel.add(beepCheckBox);

		historyCheckBox.setText(ResourceManager
				.getResource("Append_simulation_history"));
		historyCheckBox.setMnemonic('a');
		resPanel.add(historyCheckBox);

		highlightCheckBox.setText(ResourceManager
				.getResource("Highlight_changed_parameters"));
		highlightCheckBox.setMnemonic('h');
		resPanel.add(highlightCheckBox);

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 2;
		gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
		optPanel.add(resPanel, gridBagConstraints);

		getContentPane().add(optPanel, java.awt.BorderLayout.CENTER);
	}

	private void initParameters() {
		threadSlider.setValue(priority);
		iterationField.setValue(iterations);

		if (single) {
			singleRadioButton.doClick();
		} else {
			continuousRadioButton.doClick();
		}

		if ((infinite)) {
			infiniteCheckBox.setSelected(infinite);
			infiniteCheckBoxActionPerformed();
		}

		beepCheckBox.setSelected(beep);
		historyCheckBox.setSelected(history);
		highlightCheckBox.setSelected(highlight);
	}

	private void singleRadioButtonActionPerformed() {
		boolean s = !singleRadioButton.isSelected();
		single = singleRadioButton.isSelected();

		iterationLabel.setEnabled(s);
		iterationField.setEditable(s);
		infiniteCheckBox.setEnabled(s);
		if (s)
			infiniteCheckBoxActionPerformed();
	}

	private void infiniteCheckBoxActionPerformed() {
		boolean s = !infiniteCheckBox.isSelected();

		iterationLabel.setEnabled(s);
		iterationField.setEditable(s);
	}

	private void okButtonActionPerformed() {
		single = singleRadioButton.isSelected();
		infinite = infiniteCheckBox.isSelected();
		iterations = iterationField.getValue();
		priority = threadSlider.getValue();
		beep = beepCheckBox.isSelected();
		history = historyCheckBox.isSelected();
		highlight = highlightCheckBox.isSelected();

		simulationPreferences.putBoolean("single", single);
		simulationPreferences.putBoolean("infinite", infinite);
		simulationPreferences.putInt("iterations", iterations);
		simulationPreferences.putInt("priority", priority);
		simulationPreferences.putBoolean("beep", beep);
		simulationPreferences.putBoolean("history", history);
		simulationPreferences.putBoolean("highlight", highlight);

		doClose(RET_OK);
	}

	private void cancelButtonActionPerformed() {
		doClose(RET_CANCEL);
	}

	/**
	 * Closes the dialog
	 */
	private void closeDialog() {
		doClose(RET_CANCEL);
	}

	private void doClose(int retStatus) {
		returnStatus = retStatus;
		setVisible(false);
		dispose();
	}

	/**
	 * Getter for property priority.
	 * 
	 * @return Value of property priority.
	 */
	public int getPriority() {
		return priority;
	}

	/**
	 * Getter for property iterations.
	 * 
	 * @return Value of property iterations.
	 */
	public int getIterations() {
		return iterations;
	}

	/**
	 * Getter for property single.
	 * 
	 * @return Value of property single.
	 */
	public boolean isSingle() {
		return single;
	}

	/**
	 * Getter for property infinite.
	 * 
	 * @return Value of property infinite.
	 */
	public boolean isInfinite() {
		return infinite;
	}

	/**
	 * Getter for property beep.
	 * 
	 * @return Value of property beep.
	 */
	public boolean isBeep() {
		return beep;
	}

	/**
	 * Getter for property history.
	 * 
	 * @return Value of property history.
	 */
	public boolean isHistory() {
		return history;
	}

	/**
	 * Getter for property highlight.
	 * 
	 * @return Value of property highlight.
	 */
	public boolean isHighlight() {
		return highlight;
	}

	private javax.swing.JPanel buttonPanel;

	private javax.swing.JPanel threadPanel;

	private javax.swing.JPanel itrPanel;

	private javax.swing.JPanel numberPanel;

	private javax.swing.JPanel resPanel;

	private javax.swing.JButton okButton;

	private javax.swing.JButton cancelButton;

	private javax.swing.JRadioButton singleRadioButton;

	private javax.swing.JLabel iterationLabel;

	private javax.swing.JPanel simPanel;

	private javax.swing.JSlider threadSlider;

	private javax.swing.JCheckBox infiniteCheckBox;

	private com.simulogic.utils.IntegerField iterationField;

	private javax.swing.JRadioButton continuousRadioButton;

	private javax.swing.JCheckBox beepCheckBox;

	private javax.swing.JCheckBox historyCheckBox;

	private javax.swing.JCheckBox highlightCheckBox;

	private javax.swing.JPanel radioPanel;

	private javax.swing.JPanel optPanel;

	private int returnStatus = RET_CANCEL;

	/**
	 * Overrides setVisible(boolean b) in <code>JDialog</code>
	 * 
	 * @param b
	 *            visibility
	 */
	public void setVisible(boolean b) {
		if (b == true) {
			setResizable(true);
			pack();
			setResizable(false);
			getRootPane().setDefaultButton(okButton);
			setLocationRelativeTo(getOwner());
		}
		initParameters();
		super.setVisible(b);
	}
}
