/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview;

import java.awt.Component;

import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.UIManager;

/**
 * Renderer for the first column in <code>DataTable</code>
 */
public class RowHeaderRenderer implements javax.swing.table.TableCellRenderer {

    private javax.swing.border.Border selectedBorder;
    private javax.swing.border.Border emptyBorder;
    private javax.swing.JLabel header;

    /**
     * Creates new RowHeaderRenderer
     */
    public RowHeaderRenderer() {
        emptyBorder = javax.swing.BorderFactory.createEtchedBorder();
        selectedBorder = javax.swing.BorderFactory.createEmptyBorder();
        header = new javax.swing.JLabel();
        header.setHorizontalAlignment(SwingConstants.CENTER);
        header.setOpaque(true);
    }

    /**
     * Returns rendering component
     * 
     * @param table data table
     * @param value current value
     * @param isSelected true if value is selected
     * @param hasFocus true if value has focus
     * @param row row index
     * @param column column index
     * @return rendering component
     */
    public java.awt.Component getTableCellRendererComponent(JTable table,
            Object value, boolean isSelected, boolean hasFocus, int row,
            int column) {
        header.setText(value.toString());

        if (value instanceof Integer) {
            if (isSelected) {
				header.setBackground(UIManager
						.getColor("TableHeader.background"));
				header.setBorder(selectedBorder);
				if (table.getColumnCount() > 1)
					table.addColumnSelectionInterval(1,
							table.getColumnCount() - 1);
			} else {
				header.setBackground(UIManager
						.getColor("TableHeader.background"));
				header.setBorder(emptyBorder);
			}
			return header;
        }
        return (Component) table.getCellRenderer(row, column);
    }

}