/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview.legend;

import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;

import com.simulogic.dataview.DataChannel;
import com.simulogic.dataview.DataView;

/**
 * Class that specify custom selection for the legend tree: channel gets
 * selected in response to the user selection on the tree
 */
public class LegendTreeSelection implements TreeSelectionListener {

    private DataView view;

    /**
     * Default constructor
     * 
     * @param v data view
     */
    public LegendTreeSelection(DataView v) {
        view = v;
    }

    /**
     * This method is invoked when tree selection changes
     * 
     * @param evt selection event
     * @see TreeSelectionListener#valueChanged(TreeSelectionEvent)
     */
    public void valueChanged(TreeSelectionEvent evt) {
        Object vold = null;
        Object vnew = null;

        DefaultMutableTreeNode node;

        try {
            node = (DefaultMutableTreeNode) evt.getOldLeadSelectionPath()
                    .getLastPathComponent();

            if (node != null) {
                vold = node.getUserObject();
                if (vold instanceof DataChannel) {
                    ((DataChannel) vold).getAttribute().setSelected(false);
                }
            }
        } catch (Exception e) {
        }

        try {
            node = (DefaultMutableTreeNode) evt.getNewLeadSelectionPath()
                    .getLastPathComponent();

            if (node != null) {
                vnew = node.getUserObject();
                if (vnew instanceof DataChannel) {
                    ((DataChannel) vnew).getAttribute().setSelected(true);
                    view.getCursors().getCursorSlider().setLocked(true);
                    view.getCursors().getCursorSlider().repaint();
                }
            }
        } catch (Exception e) {
        }

        if (vnew != vold) {
            if (vnew instanceof DataChannel) {
                view.getStorage().setSelectedChannel(((DataChannel) vnew));
            } else {
                view.getStorage().setSelectedChannel(null);
            }
            view.renderPlot();
        }

    }
}