/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview.plots;

import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;

/**
 * Plots a styled symbol in <code>SymbolPlot</code>
 */
public class PlotSymbol {

    /**
     * Array of plot symbols
     */
    public static final GeneralPath[] symbols = new GeneralPath[15];

    /**
     * Default symbol size
     */
    private static float size = 10.0f;
    
    /**
     * Symbol type
     */
    private int type;

    /**
     * Default constructor
     * 
     * @param type symbol type
     */
    public PlotSymbol(int type) {
        super();
        this.type = type;
        createsSymbols();
    }

    /**
     * Creates an array of symbol shapes
     */
    private static final void createsSymbols() {
        float ps = size;

        float x = (float) (-ps / 2.0);
        float y = (float) (-ps / 2.0);

        GeneralPath gp = new GeneralPath();
        gp.append(new Rectangle2D.Float(x, y, ps, ps), false);
        symbols[0] = gp;

        gp = new GeneralPath();
        gp.moveTo(x, y + ps / 2.0f);
        gp.lineTo(x + ps / 2.0f, y + ps);
        gp.lineTo(x + ps, y + ps / 2.0f);
        gp.lineTo(x + ps / 2.0f, y);
        gp.lineTo(x, y + ps / 2.0f);
        symbols[1] = gp;

        gp = new GeneralPath();
        gp.moveTo(x, y + ps);
        gp.lineTo(x + ps, y + ps);
        gp.lineTo(x + ps / 2.0f, y);
        gp.lineTo(x, y + ps);
        symbols[2] = gp;

        gp = new GeneralPath();
        gp.moveTo(x, y);
        gp.lineTo(x, y + ps);
        gp.lineTo(x + ps, y);
        gp.lineTo(x + ps, y + ps);
        gp.lineTo(x, y);
        symbols[3] = gp;

        gp = new GeneralPath();
        gp.moveTo(x, y);
        gp.lineTo(x + ps, y);
        gp.lineTo(x + ps / 2.0f, y + ps);
        gp.lineTo(x, y);
        symbols[4] = gp;

        gp = new GeneralPath();
        gp.moveTo(x, y);
        gp.lineTo(x + ps, y);
        gp.lineTo(x, y + ps);
        gp.lineTo(x + ps, y + ps);
        gp.lineTo(x, y);
        symbols[5] = gp;

        gp = new GeneralPath();
        gp.append(new Line2D.Float(x, y + ps / 3.0f, x + ps, y + ps / 3.0f),
                false);
        gp.append(new Line2D.Float(x, y + 2.0f * ps / 3.0f, x + ps, y + 2.0f
                * ps / 3.0f), false);
        gp.append(new Line2D.Float(x + ps / 3.0f, y, x + ps / 3.0f, y + ps),
                false);
        gp.append(new Line2D.Float(x + 2.0f * ps / 3.0f, y, x + 2.0f * ps
                / 3.0f, y + ps), false);
        symbols[6] = gp;

        gp = new GeneralPath();
        gp.append(new Line2D.Float(x + ps / 2.0f, y, x + ps / 2.0f, y + ps
                / 2.0f), false);
        gp.append(new Line2D.Float(x, y + ps, x + ps / 2.0f, y + ps / 2.0f),
                false);
        gp.append(
                new Line2D.Float(x + ps / 2.0f, y + ps / 2.0f, x + ps, y + ps),
                false);
        symbols[7] = gp;

        gp = new GeneralPath();
        gp.append(new Line2D.Float(x, y + ps / 2.0f, x + ps, y + ps / 2.0f),
                false);
        gp.append(new Line2D.Float(x + ps / 2.0f, y, x + ps / 2.0f, y + ps),
                false);
        symbols[8] = gp;

        gp = new GeneralPath();
        gp.append(new Line2D.Float(x, y, x + ps, y + ps), false);
        gp.append(new Line2D.Float(x + ps, y, x, y + ps), false);
        symbols[9] = gp;

        gp = new GeneralPath();
        gp.append(new Line2D.Float(x, y + ps / 2.0f, x + ps, y + ps / 2.0f),
                false);
        gp.append(new Line2D.Float(x + ps / 2.0f, y, x + ps / 2.0f, y + ps),
                false);
        gp.append(new Line2D.Float(x, y, x + ps, y + ps), false);
        gp.append(new Line2D.Float(x + ps, y, x, y + ps), false);
        symbols[10] = gp;

        gp = new GeneralPath();
        gp.append(new Line2D.Float(x, y, x + ps / 2.0f, y + ps / 2.0f), false);
        gp.append(new Line2D.Float(x + ps, y, x + ps / 2.0f, y + ps / 2.0f),
                false);
        gp.append(new Line2D.Float(x + ps / 2.0f, y + ps / 2.0f, x + ps / 2.0f,
                y + ps), false);
        symbols[11] = gp;

        gp = new GeneralPath();
        gp.append(new Line2D.Float(x + ps / 2.0f, y, x + ps / 2.0f, y + ps),
                false);
        symbols[12] = gp;

        gp = new GeneralPath();
        gp.append(new Line2D.Float(x, y + ps / 2.0f, x + ps, y + ps / 2.0f),
                false);
        symbols[13] = gp;

        gp = new GeneralPath();
        gp.append(new Line2D.Float(x + ps / 3.0f, y, x + 2.0f * ps / 3.0f, y
                + ps), false);
        gp.append(new Line2D.Float(x + 2.0f * ps / 3.0f, y, x + ps / 3.0f, y
                + ps), false);
        gp.append(new Line2D.Float(x, y + ps / 3.0f, x + ps, y + 2.0f * ps
                / 3.0f), false);
        gp.append(new Line2D.Float(x, y + 2.0f * ps / 3.0f, x + ps, y + ps
                / 3.0f), false);
        symbols[14] = gp;
    }

    /**
     * Getter for property size.
     * 
     * @return Value of property size.
     */
    public float getSize() {
        return size;
    }

    /**
     * Setter for property size.
     * 
     * @param size New value of property size.
     */
    public void setSize(float size) {
        PlotSymbol.size = size;
    }

    /**
     * Getter for property type.
     * 
     * @return Value of property type.
     */
    public int getType() {
        return type;
    }

    /**
     * Setter for property type.
     * 
     * @param type New value of property type.
     */
    public void setType(int type) {
        this.type = type;
    }
    
}